@@ -8,43 +8,8 @@ use Socket qw( AF_UNIX SOCK_DGRAM );
 use IO::Handle;
 
 # ABSTRACT: Wrapper for libvlc.so
-our $VERSION = '0.01'; # VERSION
+our $VERSION = '0.02'; # VERSION
 
-=head1 SYNOPSIS
-
-  use VideoLAN::LibVLC::MediaPlayer;
-  my $player= VideoLAN::LibVLC::MediaPlayer->new("cat_video.mpg");
-  $player->video_lock_cb(sub { ... });   # allocate a buffer for video decoding
-  $player->video_unlock_cb(sub { ... }); # do something with the decoded video frame
-  $player->play; # start VLC thread that decodes and generates callbacks
-  my $fh= $player->libvlc->callback_fh;
-  while (1) {
-    if (IO::Select->new($fh)->can_read) {  # set up your main loop to watch the $fh
-      $player->libvlc->callback_dispatch # fire perl callbacks
-    }
-  }
-
-=head1 DESCRIPTION
-
-This module wraps LibVLC.  LibVLC already has a very nice object-ish (but yet
-still function-based) API, so this package wraps each group of functions into
-a perl object in the logical manner.  One difficulty, however, is that LibVLC
-uses its own threads when doing video playback, and most of the "interesting"
-possibilities for using LibVLC would require you to write your own callbacks,
-which must happen in the main Perl thread.  To work around that (and also not
-force you to use this module as your main event loop) this module passes each
-VLC callback through a pipe.  This allows you to select() on that file handle
-or use an event-driven system like AnyEvent for your program's main loop, and
-still gain the parallel processing benefit of VLC running its own thread.
-
-If you're worried about the latency of VLC's playback thread needing a round-
-trip to your callbacks, you can create a double-buffering effect by returning
-two values from the first callback.  The next VLC callback event will get the
-callback result without blocking. (but it still sends another callback event,
-so the perl callback still happens and you can stay one step ahead of the VLC
-thread)
-
-=cut
 
 use Exporter::Extensible -exporter_setup => 1;
 our %EXPORT;
@@ -96,6 +61,240 @@ sub exporter_autoload_symbol {
 require XSLoader;
 XSLoader::load('VideoLAN::LibVLC', $VideoLAN::LibVLC::VERSION);
 
+
+# wrap with method in order to ignore arguments
+sub libvlc_version   { libvlc_get_version() }
+sub libvlc_changeset { libvlc_get_changeset() }
+sub libvlc_compiler  { libvlc_get_compiler() }
+
+
+sub argv { croak("read-only attribute") if @_ > 1; $_[0]{argv} }
+
+sub _update_app_id {
+	my $self= shift;
+	$self->{app_id}= ''      unless defined $self->{app_id};
+	$self->{app_version}= '' unless defined $self->{app_version};
+	$self->{app_icon}= ''    unless defined $self->{app_icon};
+	$self->libvlc_set_app_id($self->{app_id}, $self->{app_version}, $self->{app_icon});
+}
+sub app_id      { my $self= shift; if (@_) { $self->{app_id}= shift; $self->_update_app_id; } $self->{app_id} }
+sub app_version { my $self= shift; if (@_) { $self->{app_version}= shift; $self->_update_app_id; } $self->{app_version} }
+sub app_icon    { my $self= shift; if (@_) { $self->{app_icon}= shift; $self->_update_app_id; } $self->{app_icon} }
+
+
+sub _update_user_agent {
+	my $self= shift;
+	$self->{user_agent_name}= '' unless defined $self->{user_agent_name};
+	$self->{user_agent_http}= '' unless defined $self->{user_agent_http};
+	$self->libvlc_set_user_agent($self->{user_agent_name}, $self->{user_agent_http});
+}
+sub user_agent_name { my $self= shift; if (@_) { $self->{user_agent_name}= shift; $self->_update_user_agent; } $self->{user_agent_name} }
+sub user_agent_http { my $self= shift; if (@_) { $self->{user_agent_http}= shift; $self->_update_user_agent; } $self->{user_agent_http} }
+
+
+sub audio_filters { my $self= shift; $self->{audio_filters} ||= [ $self->libvlc_audio_filter_list_get ] }
+sub video_filters { my $self= shift; $self->{video_filters} ||= [ $self->libvlc_video_filter_list_get ] }
+
+sub audio_filter_list { @{ shift->audio_filters } }
+sub video_filter_list { @{ shift->video_filters } }
+
+
+sub log { my $self= shift; $self->_set_logger(@_) if @_; $self->{log} }
+sub can_redirect_log { !!$_[0]->can('libvlc_log_unset') }
+
+# identical to libvlc api, other than needing to pump the event queue to see the messages
+sub libvlc_set_log {
+	my ($self, $callback, $argument)= @_;
+	$self->_set_logger(sub { $callback->($argument, @_) });
+}
+
+sub _set_logger {
+	my ($self, $target, $options)= @_;
+	$self->can_redirect_log
+		or croak "LibVLC log redirection is not supported in this version";
+	# If target is undef, cancel the callback
+	if (!defined $target) {
+		return $self->libvlc_log_unset;
+	}
+	else {
+		$self->_event_pipe; # init file handles, if not already done
+		# If target is a coderef:
+		if (ref $target eq 'CODE') {
+			$self->{log}= $target;
+		}
+		# if target is a logger
+		elsif (ref($target)->can('info')) {
+			$self->{log}= sub {
+				my ($level, $msg, $attr)= @_;
+				$msg= join(' ', $msg, map { "$_=$attr->{$_}" } keys %$attr);
+				if ($level == LOG_LEVEL_DEBUG()) { $target->debug($msg); }
+				elsif ($level == LOG_LEVEL_NOTICE()) { $target->notice($msg); }
+				elsif ($level == LOG_LEVEL_WARNING()) { $target->warn($msg); }
+				elsif ($level >= LOG_LEVEL_ERROR()) { $target->error($msg); }
+				else { $target->warn($msg); } # in case of future log levels
+			};
+		}
+		else {
+			croak "Don't know how to log to $target";
+		}
+		my $lev= $options? $options->{level} : undef;
+		$lev= LOG_LEVEL_NOTICE() unless defined $lev;
+		# Install callback
+		weaken($self);
+		my $cb_id= $self->_register_callback(sub { $self->log->($_[0]) } );
+		$self->_libvlc_log_set($cb_id, $lev, $options->{fields} || ['*']);
+	}
+}
+
+
+sub new {
+	my $class= shift;
+	my %args= (@_ == 1 && ref($_[0]) eq 'HASH')? %{ $_[0] }
+		: (@_ == 1 && ref($_[0]) eq 'ARRAY')? ( argv => $_[0] )
+		: ((@_&1) == 0)? @_
+		: croak "Expected hashref, even-length list, or arrayref";
+	$args{argv} ||= [];
+	my $self= VideoLAN::LibVLC::libvlc_new($args{argv});
+	%$self= %args;
+	$self->_update_app_id
+		if defined $self->{app_id} or defined $self->{app_version} or defined $self->{app_icon};
+	$self->_update_user_agent
+		if defined $self->{user_agent_name} or defined $self->{user_agent_http};
+	$self->_set_logger($self->log)
+		if defined $self->{log};
+	return $self;
+}
+
+
+sub new_media {
+	my $self= shift;
+	my @attrs= (@_ & 1) == 0? @_
+		: (@_ == 1 && !ref($_[0]))? ( ($_[0] =~ m,://,? 'location' : 'path') => $_[0] )
+		: (@_ == 1 && ref($_[0]) eq 'HASH')?        %{ $_[0] }
+		: (@_ == 1 && ref($_[0]) eq 'GLOB')?        ( fh => $_[0] )
+		: (@_ == 1 && ref($_[0])->can('scheme'))?   ( location => $_[0] )
+		: (@_ == 1 && ref($_[0])->can('absolute'))? ( path => $_[0] )
+		: (@_ == 1 && ref($_[0])->can('read'))?     ( fh => $_[0] )
+		: croak "Expected hashref, even-length list, file handle, string, Path::Class, or URI";
+	require VideoLAN::LibVLC::Media;
+	VideoLAN::LibVLC::Media->new(libvlc => $self, @attrs);
+}
+
+
+sub new_media_player {
+	my $self= shift;
+	my @attrs= (@_ & 1) == 0? @_
+		: (@_ == 1 && ref($_[0]) eq 'HASH')?   %{ $_[0] }
+		: croak "Expected hashref or even-length list";
+	require VideoLAN::LibVLC::MediaPlayer;
+	VideoLAN::LibVLC::MediaPlayer->new(libvlc => $self, @attrs);
+}
+
+
+sub callback_fh { shift->_event_pipe->[0] }
+
+sub callback_dispatch {
+	my ($self)= @_;
+	# unsolved bug - I used perl recv() and it blocks.  If I use C recv() it works....
+	my $event= $self->_recv_event()
+		or return 0;
+	my $cb= $self->{_callback}{$event->{callback_id}};
+	$cb->($event) if $cb;
+	return 1;
+}
+
+sub _event_pipe {
+	$_[0]{_event_pipe} //= do {
+		socketpair(my $r, my $w, AF_UNIX, SOCK_DGRAM, 0)
+			or die "socketpair: $!";
+		$r->blocking(0);
+		# pass file handles to XS
+		$_[0]->_set_event_pipe(fileno($r), fileno($w));
+		[$r, $w];
+	}
+}
+
+# REMINDER: Be sure to use weak references in callbacks, so that VLC instance
+# doens't end up holding onto sub-resources.
+sub _register_callback {
+	my ($self, $callback)= @_;
+	my $id= 0xFFFF & ($_[0]{_next_cb_id} ||= 1)++;
+	if ($self->{callback}{$id}) {
+		# extreme circumstances, the $id has wrapped around
+		keys %{ $self->{callback} } < 0xFFFF
+			or croak "Max callbacks reached";
+		$id= 0xFFFF & $_[0]{_next_cb_id}++
+			while $self->{callback}{$id};
+	}
+	$self->{_callback}{$id}= $callback;
+	return $id;
+}
+
+sub _unregister_callback {
+	my ($self, $id)= @_;
+	delete $self->{_callback}{$id};
+}
+
+sub libvlc_video_set_callbacks {
+	my ($player, $lock_cb, $unlock_cb, $display_cb, $opaque)= @_;
+	$player->set_video_callbacks(lock => $lock_cb, unlock => $unlock_cb, display => $display_cb, opaque => $opaque);
+}
+sub libvlc_video_set_format_callbacks {
+	my ($player, $format_cb, $cleanup_cb)= @_;
+	$player->set_video_callbacks(format => $format_cb, cleanup => $cleanup_cb);
+}
+
+
+1;
+
+__END__
+
+=pod
+
+=encoding UTF-8
+
+=head1 NAME
+
+VideoLAN::LibVLC - Wrapper for libvlc.so
+
+=head1 VERSION
+
+version 0.02
+
+=head1 SYNOPSIS
+
+  use VideoLAN::LibVLC::MediaPlayer;
+  my $player= VideoLAN::LibVLC::MediaPlayer->new("cat_video.mpg");
+  $player->video_lock_cb(sub { ... });   # allocate a buffer for video decoding
+  $player->video_unlock_cb(sub { ... }); # do something with the decoded video frame
+  $player->play; # start VLC thread that decodes and generates callbacks
+  my $fh= $player->libvlc->callback_fh;
+  while (1) {
+    if (IO::Select->new($fh)->can_read) {  # set up your main loop to watch the $fh
+      $player->libvlc->callback_dispatch # fire perl callbacks
+    }
+  }
+
+=head1 DESCRIPTION
+
+This module wraps LibVLC.  LibVLC already has a very nice object-ish (but yet
+still function-based) API, so this package wraps each group of functions into
+a perl object in the logical manner.  One difficulty, however, is that LibVLC
+uses its own threads when doing video playback, and most of the "interesting"
+possibilities for using LibVLC would require you to write your own callbacks,
+which must happen in the main Perl thread.  To work around that (and also not
+force you to use this module as your main event loop) this module passes each
+VLC callback through a pipe.  This allows you to select() on that file handle
+or use an event-driven system like AnyEvent for your program's main loop, and
+still gain the parallel processing benefit of VLC running its own thread.
+
+If you're worried about the latency of VLC's playback thread needing a round-
+trip to your callbacks, you can create a double-buffering effect by returning
+two values from the first callback.  The next VLC callback event will get the
+callback result without blocking. (but it still sends another callback event,
+so the perl callback still happens and you can stay one step ahead of the VLC
+thread)
+
 =head1 CONSTANTS
 
 This module can export constants used by LibVLC, however I renamed them a bit
@@ -156,13 +355,6 @@ Precise revision-control version of LibVLC.  This is a package attribute.
 
 Compiler used to create LibVLC.  This is a package attribute.
 
-=cut
-
-# wrap with method in order to ignore arguments
-sub libvlc_version   { libvlc_get_version() }
-sub libvlc_changeset { libvlc_get_changeset() }
-sub libvlc_compiler  { libvlc_get_compiler() }
-
 =head2 argv
 
 A copy of the argv that you passed to the constructor.  Read-only.
@@ -180,21 +372,6 @@ The version of your application.  Defaults to empty string if you assign an app_
 
 The name of the icon for your application.  Defaults to empty string if you assign an app_id or app_version.
 
-=cut
-
-sub argv { croak("read-only attribute") if @_ > 1; $_[0]{argv} }
-
-sub _update_app_id {
-	my $self= shift;
-	$self->{app_id}= ''      unless defined $self->{app_id};
-	$self->{app_version}= '' unless defined $self->{app_version};
-	$self->{app_icon}= ''    unless defined $self->{app_icon};
-	$self->libvlc_set_app_id($self->{app_id}, $self->{app_version}, $self->{app_icon});
-}
-sub app_id      { my $self= shift; if (@_) { $self->{app_id}= shift; $self->_update_app_id; } $self->{app_id} }
-sub app_version { my $self= shift; if (@_) { $self->{app_version}= shift; $self->_update_app_id; } $self->{app_version} }
-sub app_icon    { my $self= shift; if (@_) { $self->{app_icon}= shift; $self->_update_app_id; } $self->{app_icon} }
-
 =head2 user_agent_name
 
 A human-facing description of your application as a user agent for web requests.
@@ -203,17 +380,6 @@ A human-facing description of your application as a user agent for web requests.
 
 A HTTP UserAgent string for your application.
 
-=cut
-
-sub _update_user_agent {
-	my $self= shift;
-	$self->{user_agent_name}= '' unless defined $self->{user_agent_name};
-	$self->{user_agent_http}= '' unless defined $self->{user_agent_http};
-	$self->libvlc_set_user_agent($self->{user_agent_name}, $self->{user_agent_http});
-}
-sub user_agent_name { my $self= shift; if (@_) { $self->{user_agent_name}= shift; $self->_update_user_agent; } $self->{user_agent_name} }
-sub user_agent_http { my $self= shift; if (@_) { $self->{user_agent_http}= shift; $self->_update_user_agent; } $self->{user_agent_http} }
-
 =head2 audio_filters
 
 An arrayref of all audio filter modules built into LibVLC.
@@ -230,14 +396,6 @@ An arrayref of all video filter modules built into LibVLC.
 
 List accessor for video_filters.
 
-=cut
-
-sub audio_filters { my $self= shift; $self->{audio_filters} ||= [ $self->libvlc_audio_filter_list_get ] }
-sub video_filters { my $self= shift; $self->{video_filters} ||= [ $self->libvlc_video_filter_list_get ] }
-
-sub audio_filter_list { @{ shift->audio_filters } }
-sub video_filter_list { @{ shift->video_filters } }
-
 =head2 can_redirect_log
 
 Whether or not this version of libvlc supports redirecting the log.
@@ -271,55 +429,6 @@ the callback.  Available options are:
 Note that logging can happen from other threads, so you won't see the messages until
 you call L</callback_dispatch>.
 
-=cut
-
-sub log { my $self= shift; $self->_set_logger(@_) if @_; $self->{log} }
-sub can_redirect_log { !!$_[0]->can('libvlc_log_unset') }
-
-# identical to libvlc api, other than needing to pump the event queue to see the messages
-sub libvlc_set_log {
-	my ($self, $callback, $argument)= @_;
-	$self->_set_logger(sub { $callback->($argument, @_) });
-}
-
-sub _set_logger {
-	my ($self, $target, $options)= @_;
-	$self->can_redirect_log
-		or croak "LibVLC log redirection is not supported in this version";
-	# If target is undef, cancel the callback
-	if (!defined $target) {
-		return $self->libvlc_log_unset;
-	}
-	else {
-		$self->_event_pipe; # init file handles, if not already done
-		# If target is a coderef:
-		if (ref $target eq 'CODE') {
-			$self->{log}= $target;
-		}
-		# if target is a logger
-		elsif (ref($target)->can('info')) {
-			$self->{log}= sub {
-				my ($level, $msg, $attr)= @_;
-				$msg= join(' ', $msg, map { "$_=$attr->{$_}" } keys %$attr);
-				if ($level == LOG_LEVEL_DEBUG()) { $target->debug($msg); }
-				elsif ($level == LOG_LEVEL_NOTICE()) { $target->notice($msg); }
-				elsif ($level == LOG_LEVEL_WARNING()) { $target->warn($msg); }
-				elsif ($level >= LOG_LEVEL_ERROR()) { $target->error($msg); }
-				else { $target->warn($msg); } # in case of future log levels
-			};
-		}
-		else {
-			croak "Don't know how to log to $target";
-		}
-		my $lev= $options? $options->{level} : undef;
-		$lev= LOG_LEVEL_NOTICE() unless defined $lev;
-		# Install callback
-		weaken($self);
-		my $cb_id= $self->_register_callback(sub { $self->log->($_[0]) } );
-		$self->_libvlc_log_set($cb_id, $lev, $options->{fields} || ['*']);
-	}
-}
-
 =head1 METHODS
 
 =head2 new
@@ -337,26 +446,6 @@ debugging, since they can differ by version and by platform.
 The returned object is based on a hashref, and the libvlc pointer is magically
 attached.
 
-=cut
-
-sub new {
-	my $class= shift;
-	my %args= (@_ == 1 && ref($_[0]) eq 'HASH')? %{ $_[0] }
-		: (@_ == 1 && ref($_[0]) eq 'ARRAY')? ( argv => $_[0] )
-		: ((@_&1) == 0)? @_
-		: croak "Expected hashref, even-length list, or arrayref";
-	$args{argv} ||= [];
-	my $self= VideoLAN::LibVLC::libvlc_new($args{argv});
-	%$self= %args;
-	$self->_update_app_id
-		if defined $self->{app_id} or defined $self->{app_version} or defined $self->{app_icon};
-	$self->_update_user_agent
-		if defined $self->{user_agent_name} or defined $self->{user_agent_http};
-	$self->_set_logger($self->log)
-		if defined $self->{log};
-	return $self;
-}
-
 =head2 new_media
 
   my $media= $vlc->new_media( $path );
@@ -373,37 +462,10 @@ three you intended.
 You an instead pass a hash or hashref, and then it just passes them
 along to the Media constructor.
 
-=cut
-
-sub new_media {
-	my $self= shift;
-	my @attrs= (@_ & 1) == 0? @_
-		: (@_ == 1 && !ref($_[0]))? ( ($_[0] =~ m,://,? 'location' : 'path') => $_[0] )
-		: (@_ == 1 && ref($_[0]) eq 'HASH')?        %{ $_[0] }
-		: (@_ == 1 && ref($_[0]) eq 'GLOB')?        ( fh => $_[0] )
-		: (@_ == 1 && ref($_[0])->can('scheme'))?   ( location => $_[0] )
-		: (@_ == 1 && ref($_[0])->can('absolute'))? ( path => $_[0] )
-		: (@_ == 1 && ref($_[0])->can('read'))?     ( fh => $_[0] )
-		: croak "Expected hashref, even-length list, file handle, string, Path::Class, or URI";
-	require VideoLAN::LibVLC::Media;
-	VideoLAN::LibVLC::Media->new(libvlc => $self, @attrs);
-}
-
 =head2 new_media_player
 
   my $player= $vlc->new_media_player();
 
-=cut
-
-sub new_media_player {
-	my $self= shift;
-	my @attrs= (@_ & 1) == 0? @_
-		: (@_ == 1 && ref($_[0]) eq 'HASH')?   %{ $_[0] }
-		: croak "Expected hashref or even-length list";
-	require VideoLAN::LibVLC::MediaPlayer;
-	VideoLAN::LibVLC::MediaPlayer->new(libvlc => $self, @attrs);
-}
-
 =head2 callback_fh
 
 The file handle of the read-end of the callback pipe.  Listen on this file handle
@@ -419,60 +481,15 @@ API is to either call L</wait_callback> or perform your own wait on the file
 handle, and then call this method to unpack the arguments and deliver them to
 the callback.
 
+=head1 AUTHOR
+
+Michael Conrad <mike@nrdvana.net>
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2019 by Michael Conrad.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
 =cut
-
-sub callback_fh { shift->_event_pipe->[0] }
-
-sub callback_dispatch {
-	my ($self)= @_;
-	# unsolved bug - I used perl recv() and it blocks.  If I use C recv() it works....
-	my $event= $self->_recv_event()
-		or return 0;
-	my $cb= $self->{_callback}{$event->{callback_id}};
-	$cb->($event) if $cb;
-	return 1;
-}
-
-sub _event_pipe {
-	$_[0]{_event_pipe} //= do {
-		socketpair(my $r, my $w, AF_UNIX, SOCK_DGRAM, 0)
-			or die "socketpair: $!";
-		$r->blocking(0);
-		# pass file handles to XS
-		$_[0]->_set_event_pipe(fileno($r), fileno($w));
-		[$r, $w];
-	}
-}
-
-# REMINDER: Be sure to use weak references in callbacks, so that VLC instance
-# doens't end up holding onto sub-resources.
-sub _register_callback {
-	my ($self, $callback)= @_;
-	my $id= 0xFFFF & ($_[0]{_next_cb_id} ||= 1)++;
-	if ($self->{callback}{$id}) {
-		# extreme circumstances, the $id has wrapped around
-		keys %{ $self->{callback} } < 0xFFFF
-			or croak "Max callbacks reached";
-		$id= 0xFFFF & $_[0]{_next_cb_id}++
-			while $self->{callback}{$id};
-	}
-	$self->{_callback}{$id}= $callback;
-	return $id;
-}
-
-sub _unregister_callback {
-	my ($self, $id)= @_;
-	delete $self->{_callback}{$id};
-}
-
-sub libvlc_video_set_callbacks {
-	my ($player, $lock_cb, $unlock_cb, $display_cb, $opaque)= @_;
-	$player->set_video_callbacks(lock => $lock_cb, unlock => $unlock_cb, display => $display_cb, opaque => $opaque);
-}
-sub libvlc_video_set_format_callbacks {
-	my ($player, $format_cb, $cleanup_cb)= @_;
-	$player->set_video_callbacks(format => $format_cb, cleanup => $cleanup_cb);
-}
-
-
-1;
